import numpy as np
import emoji
import tile
from numpy.typing import NDArray
#from typing import Any

class board:
    def __init__(self):
        '''
        colorcodes:
        river = 0
        forest = 1
        mountain = 2
        desert = 3
        swamp = 4
        island = 5
        '''
        row1 = [0,1,3,4,1,2,5,3,0,2]
        row2 = [0,4,5,0,0,3,1,0,3,1]
        row3 = [4,0,2,0,1,0,4,2,0,5]
        row4 = [3,0,0,4,2,0,5,0,2,4]
        row5 = [1,5,3,2,5,3,0,0,1,3]
        row6 = [2,4,1,0,0,0,4,3,2,5]
        row7 = [0,5,0,0,2,1,0,5,0,4]
        row8 = [0,0,5,4,3,4,0,0,0,1]
        row9 = [4,1,3,2,1,5,2,3,5,0]
        self.colors = np.array([row1, row2, row3, row4, row5, row6, row7, row8, row9])

 
        self.tiles: NDArray[tile.tile] = np.empty([9,10], dtype = tile.tile)
        #self.tiles = np.empty([9,10], dtype = tile.tile)
        for i in range(9):
            for j in range(10):
                self.tiles[i][j] = tile.tile(i, j, self.colors[i][j])

        pos_bridges = [[[1,1],[2,0]],
                       [[2,2],[3,0]],
                       [[2,2],[4,2]],
                       [[2,2],[3,3]],
                       [[1,2],[2,4]],
                       [[0,4],[2,4]],
                       [[1,5],[2,4]],
                       [[2,6],[3,4]],
                       [[3,6],[4,5]],
                       [[3,6],[5,6]],
                       [[3,6],[4,8]],
                       [[2,7],[3,8]],
                       [[1,8],[2,7]],
                       [[0,7],[1,8]],
                       [[4,5],[5,6]],
                       [[4,5],[6,5]],
                       [[5,6],[6,5]],
                       [[6,7],[7,5]],
                       [[6,7],[8,7]],
                       [[4,4],[6,4]],
                       [[5,2],[6,4]],
                       [[5,2],[7,2]],
                       [[6,1],[7,2]],
                       [[6,1],[8,1]],
                       ]
        for brid in pos_bridges:
            self.tiles[brid[0][0]][brid[0][1]].pos_bridges.add((brid[1][0], brid[1][1]))
            self.tiles[brid[1][0]][brid[1][1]].pos_bridges.add((brid[0][0], brid[0][1]))

        #find neighbors
        directions = [
            # even rows 
            [[+1,  0], [ 0, -1], [-1, -1], 
            [-1,  0], [-1, +1], [ 0, +1]],
            # odd rows 
            [[+1,  0], [+1, -1], [ 0, -1], 
            [-1,  0], [ 0, +1], [+1, +1]],
        ]

        for i in range(9):
            for j in range(10):
                for dir in range(6):
                    offset = directions[i%2][dir]
                    if i + offset[1] >= 0 and i + offset[1] <= 8 and j + offset[0] >= 0 and j + offset[0] <= 9:
                        newneighbor = (i + offset[1],j + offset[0])
                        if self.tiles[i + offset[1]][j + offset[0]].color != 0:
                            self.tiles[i][j].neighbors.add(newneighbor)
                        else:
                            self.tiles[i][j].river1.add(newneighbor)
        
        #find shipping1
        for i in range(9):
            for j in range(10):
                for k in self.tiles[i][j].river1:
                    for l in self.tiles[k[0]][k[1]].neighbors:
                        if l[0] != i or l[1] != j:
                            self.tiles[i][j].shipping1.add(l)
                    for l in self.tiles[k[0]][k[1]].river1:
                        self.tiles[i][j].river2.add(l)
        
        #find shipping2
        for i in range(9):
            for j in range(10):
                for k in self.tiles[i][j].river2:
                    for l in self.tiles[k[0]][k[1]].neighbors:
                        if l[0] != i or l[1] != j:
                            self.tiles[i][j].shipping2.add(l)
                    for l in self.tiles[k[0]][k[1]].river1:
                        self.tiles[i][j].river3.add(l)

        #find shipping2
        for i in range(9):
            for j in range(10):
                for k in self.tiles[i][j].river3:
                    for l in self.tiles[k[0]][k[1]].neighbors:
                        if l[0] != i or l[1] != j:
                            self.tiles[i][j].shipping3.add(l)

        
        a, b = 5,6
        print(self.tiles[a][b].neighbors)
        print(self.tiles[a][b].river1)
        print(self.tiles[a][b].river2)
        print(self.tiles[a][b].river3)
        print(self.tiles[a][b].shipping1)
        print(self.tiles[a][b].shipping2)
        print(self.tiles[a][b].shipping3)
        print("----")
        print(self.tiles[a][b].pos_bridges)
        print(self.tiles[a][b].bridges)

        



    def print_board(self):
        #toprint = np.empty([9,10], dtype='U10')
        for i in range(9):
            getprinted = ''
            if i%2==1:
                getprinted += ' '
            for j in range(10):
                if self.tiles[i][j].building != 0:
                    #toprint[i][j] = self.tiles[i][j].color * 10 + self.tiles[i][j].building
                    #getprinted += str(self.tiles[i][j].color * 10 + self.tiles[i][j].building)
                    numbercode = self.tiles[i][j].color * 10 + self.tiles[i][j].building
                    if numbercode == 11:
                        getprinted += emoji.emojize(':green_square:')
                    elif numbercode == 12:
                        getprinted += emoji.emojize(':green_heart:')
                    elif numbercode == 13:
                        getprinted += emoji.emojize(':check_mark_button:')                    
                    elif numbercode == 21:
                        getprinted += emoji.emojize(':red_square:')
                    elif numbercode == 22:
                        getprinted += emoji.emojize(':red_heart:')
                    elif numbercode == 23:
                        getprinted += emoji.emojize(':no_entry:')
                    elif numbercode == 31:
                        getprinted += emoji.emojize(':yellow_square:')
                    elif numbercode == 32:
                        getprinted += emoji.emojize(':yellow_heart:')
                    elif numbercode == 33:
                        getprinted += emoji.emojize(':star:')
                    elif numbercode == 41:
                        getprinted += emoji.emojize(':new_moon:')
                    elif numbercode == 42:
                        getprinted += emoji.emojize(':black_heart:')
                    elif numbercode == 43:
                        getprinted += emoji.emojize(':chess_pawn:')
                    elif numbercode == 51:
                        getprinted += emoji.emojize(':blue_square:')
                    elif numbercode == 52:
                        getprinted += emoji.emojize(':blue_heart:')
                    elif numbercode == 53:
                        getprinted += emoji.emojize(':whale:')
                    '''             elif (i==5 or i==6) and j <=8:
                    if i==5 and j == 0: numbercode = 11
                    if i==5 and j == 1: numbercode = 12
                    if i==5 and j == 2: numbercode = 13
                    if i==5 and j == 3: numbercode = 21
                    if i==5 and j == 4: numbercode = 22
                    if i==5 and j == 5: numbercode = 23
                    if i==5 and j == 6: numbercode = 31
                    if i==5 and j == 7: numbercode = 32
                    if i==5 and j == 8: numbercode = 33
                    if i==6 and j == 0: numbercode = 11
                    if i==6 and j == 1: numbercode = 12
                    if i==6 and j == 2: numbercode = 13
                    if i==6 and j == 3: numbercode = 41
                    if i==6 and j == 4: numbercode = 42
                    if i==6 and j == 5: numbercode = 43
                    if i==6 and j == 6: numbercode = 51
                    if i==6 and j == 7: numbercode = 52
                    if i==6 and j == 8: numbercode = 53
                    if numbercode == 11:
                        getprinted += emoji.emojize(':green_square:')
                    elif numbercode == 12:
                        getprinted += emoji.emojize(':green_heart:')
                    elif numbercode == 13:
                        getprinted += emoji.emojize(':check_mark_button:')                    
                    elif numbercode == 21:
                        getprinted += emoji.emojize(':red_square:')
                    elif numbercode == 22:
                        getprinted += emoji.emojize(':red_heart:')
                    elif numbercode == 23:
                        getprinted += emoji.emojize(':no_entry:')
                    elif numbercode == 31:
                        getprinted += emoji.emojize(':yellow_square:')
                    elif numbercode == 32:
                        getprinted += emoji.emojize(':yellow_heart:')
                    elif numbercode == 33:
                        getprinted += emoji.emojize(':star:')
                    elif numbercode == 41:
                        getprinted += emoji.emojize(':new_moon:')
                    elif numbercode == 42:
                        getprinted += emoji.emojize(':black_heart:')
                    elif numbercode == 43:
                        getprinted += emoji.emojize(':chess_pawn:')
                    elif numbercode == 51:
                        getprinted += emoji.emojize(':blue_square:')
                    elif numbercode == 52:
                        getprinted += emoji.emojize(':blue_heart:')
                    elif numbercode == 53:
                        getprinted += emoji.emojize(':whale:')
'''
                else:
                    if self.tiles[i][j].color == 0:
                        #toprint[i][j] = emoji.emojize(':white_circle:')
                        getprinted += emoji.emojize(':white_circle:')
                    elif self.tiles[i][j].color == 1:
                        #toprint[i][j] = emoji.emojize(':green_circle:')
                        getprinted += emoji.emojize(':green_circle:')
                    elif self.tiles[i][j].color == 2:
                        #toprint[i][j] = emoji.emojize(':red_circle:')
                        getprinted += emoji.emojize(':red_circle:')
                    elif self.tiles[i][j].color == 3:
                        #toprint[i][j] = emoji.emojize(':yellow_circle:')
                        getprinted += emoji.emojize(':yellow_circle:')
                    elif self.tiles[i][j].color == 4:
                        #toprint[i][j] = emoji.emojize(':black_circle:')
                        getprinted += emoji.emojize(':black_circle:')
                    elif self.tiles[i][j].color == 5:
                        #toprint[i][j] = emoji.emojize(':blue_circle:')
                        getprinted += emoji.emojize(':blue_circle:')
                    #else:
                        #toprint[i][j] = self.tiles[i][j].color
            print(getprinted)
        #print(toprint)